/*WBL 25 Nov 2025 example taken from
  https://github.com/rapidsai/cuspatial/tree/branch-24.12/cpp/cuproj#example

Modifications:
WBL 19 Dec 2024 Add printing expected and transform output, use wgs_to_utm_test.cu includes
WBL 18 Dec 2024 replacement based on branch-24.12 cuspatial/cpp/cuproj/tests/wgs_to_utm_test.cu TYPED_TEST(ProjectionTest, readme_example)
   also https://web.physics.utah.edu/~detar/lessons/c++/hello/node1.html
WBL 17 Dec 2024 replace with https://github.com/rapidsai/cuspatial/tree/branch-25.02/cpp/cuproj#example
*/

using namespace std;

#include <cuproj/error.hpp>
#include <cuproj/projection_factories.cuh>
#include <cuproj/vec_2d.hpp>

#include <thrust/device_vector.h>
#include <thrust/host_vector.h>

#include <proj.h>

#include <iostream>
#include <iomanip>

int main() {                            
  cout << "Start main()\n";

  using T = float;
  // Make a projection to convert WGS84 (lat, lon) coordinates to UTM zone 56S (x, y) coordinates
  auto* proj = cuproj::make_projection<cuproj::vec_2d<T>>("EPSG:4326", "EPSG:32756");

  cuproj::vec_2d<T> sydney{-33.858700, 151.214000};  // Sydney, NSW, Australia
  thrust::device_vector<cuproj::vec_2d<T>> d_in{1, sydney};
  thrust::device_vector<cuproj::vec_2d<T>> d_out(d_in.size());

  // Convert the coordinates. Works the same with a vector of many coordinates.
  proj->transform(d_in.begin(), d_in.end(), d_out.begin(), cuproj::direction::FORWARD);

  //Check answer (based on wgs_to_utm_test.cu run_cuproj_test)
  const float  expected[2] = {334784.375, 6252076};
//const double expected[2] = {334783.95448071020655, 6252075.9617414548993};
  cout << "expected " << std::setprecision(20) << expected[0] << " " << expected[1]
       << endl;
  cuproj::vec_2d<T> c_out = d_out[0];
  cout << "Device: " << std::setprecision(20) << c_out.x << " " << c_out.y << endl;
  cout << "end main().\n"; 
}
